library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity clk_dimm_module is
  port (
    system_clk : in  std_logic;
    cpu_clk    : out std_logic;
    bus_clk    : out std_logic;
    dcm_lock   : out std_logic;

    sdram_csn  : in std_logic;
    sdram_cke  : in std_logic;
    sdram_clk  : in std_logic;
    sdram_addr12 : out std_logic;
    sdram_wp   : out std_logic;
    sdram_scl  : out std_logic;
    sdram_rege : out std_logic;
    sdram_cs0n : out std_logic;
    sdram_cs1n : out std_logic;
    sdram_cs2n : out std_logic;
    sdram_cs3n : out std_logic;
    sdram_cke0 : out std_logic;
    sdram_cke1 : out std_logic;
    sdram_clk0 : out std_logic;
    sdram_clk1 : out std_logic;
    sdram_clk2 : out std_logic;
    sdram_clk3 : out std_logic;
    
    flash_mem_A : in std_logic_vector(0 to 31);
    flash_A    : out std_logic_vector(20 downto 0)
    );
end clk_dimm_module;

architecture Behavioral of clk_dimm_module is

  component dcm300
    port(
      RST_IN : in std_logic;
      CLKIN_IN : in std_logic;
      LOCKED_OUT : out std_logic;
      CLKFX_OUT : out std_logic;
      CLK0_OUT : out std_logic);
  end component;

  component dcm_sdram
    port (
      RST_IN : in std_logic;
      CLKIN_IN : in std_logic;
      LOCKED_OUT : out std_logic;
      CLK180_OUT : out std_logic;
      CLK0_OUT : out std_logic);
  end component;
  
  component DDR_Output
    port (
      clk : in std_logic;
      clk180 : in std_logic;
      d0 : in std_logic;
      d1 : in std_logic;
      ce : in std_logic;
      rst : in std_logic;
      set : in std_logic;
      q : out std_logic);
  end component;

  signal sdr_clk180 : std_logic;
  signal sdr_clk0 : std_logic;
begin
  
  sdram_addr12 <= '0';
  sdram_wp <= '0';
  sdram_scl <= '0';
  sdram_rege <= '0';
  
  flash_A <= flash_mem_A(11 to 31);

  u_dcm : dcm300
    port map(RST_IN => '0', CLKIN_IN => system_clk,
             LOCKED_OUT => dcm_lock, CLKFX_OUT => cpu_clk,
             CLK0_OUT => bus_clk);

  u_dcm_ram : dcm_sdram
    port map (						
      RST_IN     => '0',
      CLKIN_IN   => sdram_clk,
      CLK180_OUT => sdr_clk180,
      CLK0_OUT   => sdr_clk0);
  

  -- CLK0 ---------------------------------------------------------------------------
  u_ddr0 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => '1',
      d1     => '0',
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_clk0);
  -- CLK1 ---------------------------------------------------------------------------
  u_ddr1 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => '1',
      d1     => '0',
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_clk1);
  -- CLK2 ---------------------------------------------------------------------------
  u_ddr2 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => '1',
      d1     => '0',
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_clk2);
  -- CLK3 ---------------------------------------------------------------------------
  u_ddr3 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => '1',
      d1     => '0',
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_clk3);
  -- CKE0 ---------------------------------------------------------------------------
  u_ddr4 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => sdram_cke,
      d1     => sdram_cke,
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_cke0);
  -- CKE1 ---------------------------------------------------------------------------
  u_ddr5 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => sdram_cke,
      d1     => sdram_cke,
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_cke1);

  -- CS0 ---------------------------------------------------------------------------
  u_ddr6 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => sdram_csn,
      d1     => sdram_csn,
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_cs0n);
  -- CS1 ---------------------------------------------------------------------------
  u_ddr7 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => sdram_csn,
      d1     => sdram_csn,
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_cs1n);
  -- CS2 ---------------------------------------------------------------------------
  u_ddr8 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => sdram_csn,
      d1     => sdram_csn,
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_cs2n);
  -- CS3 ---------------------------------------------------------------------------
  u_ddr9 : DDR_Output
    port map (
      clk    => sdr_clk0,
      clk180 => sdr_clk180,
      d0     => sdram_csn,
      d1     => sdram_csn,
      ce     => '1',
      rst    => '0',
      set    => '0',
      q      => sdram_cs3n);

end Behavioral;
